package com.tencent.iot.speech.app.asr.flash_file;

import android.Manifest;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.graphics.Color;
import android.net.Uri;
import android.os.Bundle;
import android.provider.Settings;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;

import android.view.View;

import android.widget.TextView;
import android.widget.Toast;

import com.tencent.cloud.qcloudasrsdk.models.QCloudFlashRecognitionParams;
import com.tencent.cloud.qcloudasrsdk.recognizer.QCloudFlashRecognizer;
import com.tencent.cloud.qcloudasrsdk.recognizer.QCloudFlashRecognizerListener;
import com.tencent.iot.speech.app.DemoConfig;
import com.tencent.iot.speech.app.R;
import com.tencent.iot.speech.asr.listener.MessageListener;
import com.victor.loading.rotate.RotateLoading;

import java.io.IOException;
import java.io.InputStream;

public class FileFlashRecognizeActivity extends AppCompatActivity implements QCloudFlashRecognizerListener, MessageListener {

    private QCloudFlashRecognizer fileFlashRecognizer;

    private int REQUEST_CODE = 1002;


    //磁盘读写权限 麦克风录音权限
    String[] permiss = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE,
            Manifest.permission.WRITE_EXTERNAL_STORAGE,
            Manifest.permission.RECORD_AUDIO};

    private void showLoading(boolean showLoading) {
        RotateLoading rotateLoading = findViewById(R.id.rotateloading);
        if (showLoading) {
            rotateLoading.start();
        }
        else {
            rotateLoading.stop();
        }
    }
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        setContentView(R.layout.activity_file_flash_recognizer);

        RotateLoading rotateLoading = findViewById(R.id.rotateloading);
        rotateLoading.setLoadingColor(Color.parseColor("#FF79CD6E"));
        if(!checkMyPermission(permiss)){
            ActivityCompat.requestPermissions(this,permiss, REQUEST_CODE);
        }

        if (fileFlashRecognizer == null) {
                /**直接鉴权**/
            fileFlashRecognizer = new QCloudFlashRecognizer(DemoConfig.apppId, DemoConfig.secretId, DemoConfig.secretKey);

               /**使用临时密钥鉴权
                * * 1.通过sts 获取到临时证书 （secretId secretKey  token） ,此步骤应在您的服务器端实现，见https://cloud.tencent.com/document/product/598/33416
                *   2.通过临时密钥调用接口
                * **/
//            fileFlashRecognizer = new QCloudFlashRecognizer(DemoConfig.apppId, "临时secretId", "临时secretKey","对应的token");

            fileFlashRecognizer.setCallback(this);
        }
        System.out.println(Thread.currentThread());

        findViewById(R.id.recognize_flash_file).setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                InputStream is = null;
                try {
                    AssetManager am = getResources().getAssets();
                    is = am.open("test2.mp3");
                    int length = is.available();
                    byte[] audioData = new byte[length];
                    is.read(audioData);

                    QCloudFlashRecognitionParams params = (QCloudFlashRecognitionParams) QCloudFlashRecognitionParams.defaultRequestParams();

                    /**支持传音频文件数据或者音频文件路径，如果同时调用setData和setPath，sdk内将忽略setPath
                     *  音频文件支持100M以内的文件，如果使用setData直接传音频文件数据，需要避免数据过大引发OOM，大文件建议传路径
                     *  setVoiceFormat必须正确，否则服务器端将无法解析
                     *  参数解释详解API文档https://cloud.tencent.com/document/product/1093/52097
                     * **/
                    params.setData(audioData);
//                    params.setPath("/sdcard/test2.mp3");
                    params.setVoiceFormat("mp3"); //音频格式。支持 wav、pcm、ogg-opus、speex、silk、mp3、m4a、aac。

                    /**以下参数不设置将使用默认值**/
//                    params.setEngineModelType("16k_zh");//引擎模型类型,默认16k_zh。8k_zh：8k 中文普通话通用；16k_zh：16k 中文普通话通用；16k_zh_video：16k 音视频领域。
//                    params.setFilterDirty(0);// 0 ：默认状态 不过滤脏话 1：过滤脏话
//                    params.setFilterModal(0);// 0 ：默认状态 不过滤语气词  1：过滤部分语气词 2:严格过滤
//                    params.setFilterPunc(0);// 0 ：默认状态 不过滤句末的句号 1：滤句末的句号
//                    params.setConvertNumMode(1);//1：默认状态 根据场景智能转换为阿拉伯数字；0：全部转为中文数字。
//                    params.setSpeakerDiarization(0); //是否开启说话人分离（目前支持中文普通话引擎），默认为0，0：不开启，1：开启。
//                    params.setFirstChannelOnly(1); //是否只识别首个声道，默认为1。0：识别所有声道；1：识别首个声道。
//                    params.setWordInfo(0); //是否显示词级别时间戳，默认为0。0：不显示；1：显示，不包含标点时间戳，2：显示，包含标点时间戳。


                    long ret = 0;
                    ret = fileFlashRecognizer.recognize(params);

                    if (ret >= 0) {
                        showLoading(true);
                    }
                } catch (IOException e) {
                    showLoading(false);
                    onMessage("录音文件不存在");
                    e.printStackTrace();
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    try {
                        if (is != null) is.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        });
    }

    private boolean checkMyPermission(String[] permiss){
        if(permiss !=null && permiss.length > 0 ){
            for(String per : permiss) {
                if (ContextCompat.checkSelfPermission(this, per) != PackageManager.PERMISSION_GRANTED){
                    return false;
                }
            }
        }
        return true;
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults){
        if (requestCode == REQUEST_CODE) {
            boolean isAllGranted = true;
            // 判断是否所有的权限都已经授予了
            for (int grant : grantResults) {
                if (grant != PackageManager.PERMISSION_GRANTED) {
                    isAllGranted = false;
                    break;
                }
            }
            if (isAllGranted) {
            } else {
                // 弹出对话框告诉用户需要权限的原因, 并引导用户去应用权限管理中手动打开权限按钮
                openAppDetails();
            }
        }
    }

    /**
     * 打开 APP 的详情设置
     */
    private void openAppDetails() {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setMessage("录音需要访问 “外部存储器”，请到 “应用信息 -> 权限” 中授予！");
        builder.setPositiveButton("去手动授权", new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                Intent intent = new Intent();
                intent.setAction(Settings.ACTION_APPLICATION_DETAILS_SETTINGS);
                intent.addCategory(Intent.CATEGORY_DEFAULT);
                intent.setData(Uri.parse("package:" + getPackageName()));
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
                intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY);
                intent.addFlags(Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS);
                startActivity(intent);
            }
        });
        builder.setNegativeButton("取消", null);
        builder.show();
    }

    @Override
    protected void onStart() {
        super.onStart();
    }


    //录音文件识别结果回调 ，详见api文档 https://cloud.tencent.com/document/product/1093/52097
    @Override
    public void recognizeResult(QCloudFlashRecognizer recognizer,  String result, Exception exception) {
        showLoading(false);
        TextView textView = findViewById(R.id.recognize_flash_text_view);
        if (exception != null){
            textView.setText(exception.getLocalizedMessage());
        }else {
            textView.setText(result);
        }

    }

    @Override
    public void onMessage(final String msg) {
        runOnUiThread(new Runnable() {
            public void run() {
                Toast.makeText(FileFlashRecognizeActivity.this, msg, Toast.LENGTH_SHORT).show();
            }
        });
    }
}
